;;; -*- mode: lisp; syntax: common-lisp; base: 10; package: NET.ASERVE -*-
;;
;; allegroserve-sjs.lisp -- Allegorserve extension for serving sjs files
;;                          as JavaScript.
;;
;; Copyright (c) 2006, Jason Kantz <jason@kantz.com>
;; All rights reserved.
;;
;;; Code:

(in-package :net.aserve)


(defpackage :net.aserve
  (:export #:sjs-directory-entity-publisher
	   #:sjs-entity
	   #:publish-sjs))

(defclass sjs-entity (file-entity) ())

(defun publish-sjs (&key (host nil host-p) port path class
                    (server *wserver*) locator remove authorizer filename
                    (timeout #+io-timeout #.(* 100 24 60 60)
                             #-io-timeout nil)
                    (content-type "text/javascript") plist)
  ;; publish a lisp javascript file
  (if* (null locator) 
       then (setq locator (find-locator :exact server)))
  (if* remove
       then (unpublish-entity locator path
                              host
                              host-p)
       (return-from publish-sjs nil))
  (let* ((hval (convert-to-vhosts (if* (and host (atom host))
                                       then (list host)
                                       else host)
				  server))
         (ent (make-instance (or class 'sjs-entity)
                             :host hval
                             :port port
                             :path path
                             :authorizer authorizer
                             :file filename
                             :content-type content-type
                             :timeout timeout
                             :plist plist)))
    (publish-entity ent locator path hval)
    ent))

(defun sjs-directory-entity-publisher (req ent realname info
                                       &key (suffixes '("sjs"))
                                       (sjs-content-type "text/javascript"))
  ;; publish all files ending in .sjs as sjs files
  ;; otherwise do the standard-directory-entity-publisher thing
  ;;
  (multiple-value-bind (root tail name type) (split-namestring realname)
    (declare (ignore root tail name))
    (if* (member type suffixes :test #'equalp)
         then (multiple-value-bind (content-type local-authorizer) (standard-access-file-reader realname info)
                (publish-sjs :path (request-decoded-uri-path req)
                             :host (host ent)
                             :filename realname
                             :authorizer (or local-authorizer
                                             (entity-authorizer ent))
                             :content-type (or content-type 
                                               sjs-content-type)
                             :timeout (entity-timeout ent)
                             :plist (list :parent ent) ; who spawned us
                             ))
         else (standard-directory-entity-publisher req ent realname info))))

(defmethod process-entity :around ((req http-request) (ent sjs-entity))
  (jsgen::js-compile-file (file ent))
  (setf (slot-value ent 'file) (make-pathname :type "js" :defaults (file ent)))
  (call-next-method)
  (setf (slot-value ent 'file) (make-pathname :type "sjs" :defaults (file ent))))


;;; allegroserve-sjs.lisp ends here