;;; -*- mode: lisp; syntax: common-lisp; base: 10; package: JSGEN -*-
;;
;; read-enable.lisp -- Macros to enable and disable readtables.
;;
;; Copyright (c) 2006, Jason Kantz <jason@kantz.com>
;; All rights reserved.
;;
;;; Code:

(in-package "JSGEN")


(defmacro defreadtable (name &body body)
  `(progn
    (defparameter ,name (copy-readtable nil))
    ,@body
    ,name))

(defvar *readtable-stack* nil)

(defmacro push-readtable (readtable)
  "Enable a reader syntax by copyting READTABLE into *READTABLE*."
  `(eval-when (:compile-toplevel :load-toplevel :execute)
    (push (copy-readtable) *readtable-stack*)
    (copy-readtable ,readtable *readtable*)))

(defmacro pop-readtable ()
  "Restore readtable which was active before last call to ENABLE-READTABLE.
If there was no such call, the standard readtable is used."
  (declare (ignore readtable))
  `(eval-when (:compile-toplevel :load-toplevel :execute)
    (if *readtable-stack*
        (setq *readtable* (pop *readtable-stack*))
        (setq *readtable* (copy-readtable nil)))))


;;; read-enable.lisp ends here